local buffContainer = nil

local buffs = {}
local raid = {}

local throttleCounter = 0

local status = 
{
	enabled = true,
	active = false,
	throttled = false,
	waiting = false
}

local options =
{
	buttonSize = 20,
	buttonGap = 5,
	throttle = 1.0
}

function OnLoad()
	
	local buffArgs = {
		{
			spellName = "Power Word: Fortitude",
			groupSpellName = "Prayer of Fortitude",
			spellIcon = [[Interface\Icons\Spell_Holy_WordFortitude]],
		},
		{
			spellName = "Divine Spirit",
			groupSpellName = "Prayer of Spirit",
			spellIcon = [[Interface\Icons\Spell_Holy_DivineSpirit]],
		},
		{
			spellName = "Arcane Intellect",
			groupSpellName = "Arcane Brilliance",
			spellIcon = [[Interface\Icons\Spell_Holy_MagicalSentry]],
		},
		{
			spellName = "Mark of the Wild",
			groupSpellName = "Gift of the Wild",
			spellIcon = [[Interface\Icons\Spell_Nature_Regeneration]],
		},
	}
	
	buffContainer = CreateFrame("Button", nil, UIParent, nil)
	buffContainer:RegisterForDrag("LeftButton")
	buffContainer:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	buffContainer:SetBackdrop(
		{ 
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
			tile = true, tileSize = 16, edgeSize = 16, 
			insets = { left = 0, right = 0, top = 0, bottom = 0 }
		}
	)
	buffContainer:SetBackdropColor(0,0,0,1)
	
	buffContainer:SetScript("OnEvent", EventSwitch)
	
	buffContainer:SetMovable(true)
	buffContainer:SetScript("OnDragStart", OnDragStart)
	buffContainer:SetScript("OnDragStop", OnDragStop)
	
	CreateFrame("GameTooltip", "buffTooltip", nil, "GameTooltipTemplate")
	buffTooltip:SetOwner(buffContainer, "ANCHOR_NONE" )
	buffTooltip.frame = nil
	
	for index = 1, #buffArgs, 1 do
		local args = buffArgs[index]
		--local buffTemp = {}

		local buffTemp = CreateFrame("Button", nil, buffContainer, "SecureActionButtonTemplate")
		
		buffTemp.spellName = args.spellName
		buffTemp.groupSpellName = args.groupSpellName
		buffTemp.pointer = 1
		buffTemp.missing = {}
		
		buffTemp:RegisterForClicks("AnyUp")
		buffTemp:SetAttribute("ctrl-unit*", "target")
		buffTemp:SetAttribute("shift-type*", "message")
		function buffTemp.message()
			countText = "Missing [" .. buffTemp.spellName .. "]: " .. #buffTemp.missing
			SendChatMessage(countText, "RAID")
			if (#buffTemp.missing > 0) then
				names = ""
				for index = 1, #buffTemp.missing, 1 do
					names = names .. buffTemp.missing[index] .. ", "
				end
				names = strsub(names, 1, -3)
				SendChatMessage(names, "RAID")
			end
		end

		buffTemp:RegisterEvent("OnEnter")
		buffTemp:SetScript("OnEnter", ShowTooltip)
		buffTemp:RegisterEvent("OnLeave")
		buffTemp:SetScript("OnLeave", HideTooltip)
		
		buffTemp:SetScript("PreClick", ButtonPreClick)
		buffTemp:SetScript("PostClick", ButtonPostClick)

		if (index == 1) then
			buffTemp:SetPoint("TOPLEFT", buffContainer, "TOPLEFT", options.buttonGap, -options.buttonGap)
		else
			buffTemp:SetPoint("TOPLEFT", buffs[index-1], "TOPRIGHT", options.buttonGap, 0)
		end
		
		buffTemp:SetBackdrop( {
					bgFile = args.spellIcon,
					tile = false,
					insets = {left = 0, right = 0, top = 0, bottom = 0}
		})
		buffTemp:SetBackdropBorderColor(1, 1, 1, 0)
		buffTemp.label = buffTemp:CreateFontString("labelTemp", "OVERLAY", "GameFontNormal")
		buffTemp.label:SetFont("Fonts\FRIZQT__.TTF", 8)
		buffTemp.label:SetTextColor(0, 1, 0, 1)
		buffTemp.label:SetPoint("CENTER", buffTemp, "CENTER", 0, 0)
		buffTemp.label:SetText(buffTemp.count)
		
		buffs[index] = buffTemp
	end
	
	ResizeButtons()
	if (status.enabled) then
		SetEnabled()
	end

end

function ResizeButtons()
	if not (InCombatLockdown()) then
		for index = 1, #buffs, 1 do
			buffs[index]:SetWidth(options.buttonSize)
			buffs[index]:SetHeight(options.buttonSize)
		end
		buffContainer:SetWidth(#buffs * (options.buttonSize + options.buttonGap) + options.buttonGap)
		buffContainer:SetHeight(options.buttonSize + (2 * options.buttonGap))
	end
end

function UpdateBuffs()
	if (status.throttled) then
		SetWaiting()
	else
		for _, buff in pairs(buffs) do
			for name in pairs(buff.missing) do
				buff.missing[name] = nil
			end
		end
		raidIndex = 1
		repeat
		raidMem = "raid" .. raidIndex
			raidMemName, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(raidIndex)
			for _, buff in pairs(buffs) do
				buffIndex = 1
				buffFound = false
				repeat
					buffName, rank, iconTexture, count, duration, timeLeft =  UnitBuff(raidMem, buffIndex)
						if (buffName == buff.spellName or buffName == buff.groupSpellName) then
							buffFound = true
						end
					buffIndex = buffIndex + 1
				until (not buffName)
				if not (buffFound) then
					table.insert(buff.missing, raidMemName)
				end
			end
			raidIndex = raidIndex + 1
		until (not raidMemName)
		for _, buff in pairs(buffs) do
			buff.label:SetText(#buff.missing)
			ValidatePointer(buff)
		end
		
		SetThrottled()
		SetNotWaiting()
		if (buffTooltip.frame) then
			ShowTooltip(buffTooltip.frame)
		end
	end
	
end

function OnUpdate(self, elapsed)
	throttleCounter = throttleCounter + elapsed
	if (status.throttled and (throttleCounter > options.throttle)) then
		SetUnthrottled()
	end
end

function ShowTooltip(button)
	buffTooltip.frame = button
	buffTooltip:SetOwner(buffContainer, "ANCHOR_NONE" )
	buffTooltip:SetPoint("BOTTOMLEFT", button, "TOPLEFT", 0, 0)
	buffTooltip:AddLine(button.spellName)
	for _, val in pairs(button.missing) do
		buffTooltip:AddLine(val)
	end
	buffTooltip:Show()
end

function HideTooltip()
	buffTooltip.frame = nil
	buffTooltip:ClearLines()
	buffTooltip:Hide()
end

function ButtonPreClick(button)
	Debug(button.pointer)
	ValidatePointer(button)
	button:SetAttribute("*type*", "spell")
	button:SetAttribute("*spell*", button.spellName)
	button:SetAttribute("*unit*", button.missing[button.pointer])
	button.pointer = button.pointer + 1
end

function ButtonPostClick(button)
	button:SetAttribute("*type*", nil)
	button:SetAttribute("*spell*", nil)
	button:SetAttribute("*unit*", nil)
end

function ValidatePointer(button)
	if (#button.missing == 0) then
		button.pointer = 0
	elseif (button.pointer > #button.missing) then
		button.pointer = 1
	end
end

function OnDragStart(frame)
	frame:StartMoving()
end

function OnDragStop(frame)
	frame:StopMovingOrSizing()
end

function SetEnabled()
	status.enabled=true
	buffContainer:SetScript("OnUpdate", OnUpdate)
	buffContainer:RegisterEvent("RAID_ROSTER_UPDATE")
	buffContainer:RegisterEvent("PLAYER_ENTERING_WORLD")
	buffContainer:RegisterEvent("PLAYER_REGEN_ENABLED")
	buffContainer:RegisterEvent("PLAYER_REGEN_DISABLED")
end

function SetDisabled()
	status.enabled=false
	SetDeactivated()
	eventTracker:UnregisterAllEvents()
end

function SetActive()
	status.active = true
	UpdateBuffs()
	buffContainer:RegisterEvent("UNIT_AURA")
	buffContainer:Show()
end

function SetInactive()
	status.active = false
	buffContainer:Unregisterevent("UNIT_AURA")
	buffContainer:Hide()
end

function SetThrottled()
	status.throttled = true
	throttleCounter = 0
end

function SetUnthrottled()
	status.throttled = false
	if (status.waiting) then
		UpdateBuffs()
	end
end

function SetWaiting()
	status.waiting = true
end

function SetNotWaiting()
	status.waiting = false
end

function EventSwitch(frame, event, ...)
	if (event == "RAID_ROSTER_UPDATE") or (event == "PLAYER_ENTERING_WORLD") then
		if (UnitInRaid("player") and (not status.active)) then
			SetActive()
		elseif ((not UnitInRaid("player")) and status.active) then
			SetInactive()
		end
		if (status.active) then
			UpdateBuffs()
		end
	elseif (event == "UNIT_AURA") then
		UpdateBuffs()
	end
end

function Debug(message)
	ChatFrame1:AddMessage(message)
end

OnLoad()